@@ -32,12 +32,12 @@ typedef const struct _PedDiskArchOps	PedDiskArchOps;
 #include <parted/geom.h>
 
 typedef enum {
+	PED_PARTITION_NORMAL		= 0x00,
 	PED_PARTITION_LOGICAL		= 0x01,
 	PED_PARTITION_EXTENDED		= 0x02,
 	PED_PARTITION_FREESPACE		= 0x04,
 	PED_PARTITION_METADATA		= 0x08
 } PedPartitionType;
-/* #define PED_PARTITION_PRIMARY	0 */
 
 typedef enum {
 	PED_PARTITION_BOOT=1,
