@@ -1658,7 +1658,7 @@ ped_disk_get_max_partition_geometry (PedDisk* disk, PedPartition* part,
 {
 	PedGeometry	old_geom;
 	PedGeometry*	max_geom;
-	PedConstraint*	constraint_any;
+	PedConstraint*	constraint_exact;
 
 	PED_ASSERT(disk != NULL, return NULL);
 	PED_ASSERT(part != NULL, return NULL);
@@ -1669,10 +1669,10 @@ ped_disk_get_max_partition_geometry (PedDisk* disk, PedPartition* part,
 		return NULL;
 	max_geom = ped_geometry_duplicate (&part->geom);
 
-	constraint_any = ped_constraint_any (disk->dev);
-	ped_disk_set_partition_geom (disk, part, constraint_any,
+	constraint_exact = ped_constraint_exact (&old_geom);
+	ped_disk_set_partition_geom (disk, part, constraint_exact,
 				     old_geom.start, old_geom.end);
-	ped_constraint_destroy (constraint_any);
+	ped_constraint_destroy (constraint_exact);
 
 	/* this assertion should never fail, because the old
 	 * geometry was valid
