@@ -4,7 +4,7 @@
     original version by Matt Domsch <Matt_Domsch@dell.com>
     Disclaimed into the Public Domain
 
-    Portions Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+    Portions Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
 
     EFI GUID Partition Table handling
     Per Intel EFI Specification v1.02
@@ -51,7 +51,7 @@
 #define EFI_PMBR_OSTYPE_EFI 0xEE
 #define MSDOS_MBR_SIGNATURE 0xaa55
 
-#define GPT_HEADER_SIGNATURE 0x5452415020494645
+#define GPT_HEADER_SIGNATURE 0x5452415020494645LL
 #define GPT_HEADER_REVISION_V1_00 0x00010000
 #define GPT_HEADER_REVISION_V0_99 0x00009900
 
@@ -462,9 +462,10 @@ _parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt)
 		if (ped_exception_throw (
 			PED_EXCEPTION_WARNING,
 			PED_EXCEPTION_IGNORE_CANCEL,
-			_("The format of the GPT partition table is newer "
-			  "than what Parted can recognise.  Please tell us!  "
-			  "bug-parted@gnu.org"))
+			_("The format of the GPT partition table is version "
+			  "%x, which is newer than what Parted can "
+			  "recognise.  Please tell us!  bug-parted@gnu.org"),
+			PED_CPU_TO_LE32 (gpt->Revision))
 				!= PED_EXCEPTION_IGNORE)
 			return 0;
 	}
@@ -885,6 +886,7 @@ gpt_partition_duplicate (const PedPartition* part)
 				       part->geom.start, part->geom.end);
 	if (!result)
 		goto error;
+	result->num = part->num;
 
 	if (result->type != 0)
 		return result;
