@@ -60,69 +60,71 @@ typedef struct _FatInfoSector	FatInfoSector;
 #define FAT_BOOT_CODE_LENGTH 128
 
 struct _FatBootSector {
-	__u8	boot_jump[3];	/* 00: Boot strap short or near jump */
-	__u8	system_id[8];	/* 03: system name */
-	__u16	sector_size;	/* 0b: bytes per logical sector */
-	__u8	cluster_size;	/* 0d: sectors/cluster */
-	__u16	reserved;	/* 0e: reserved sectors */
-	__u8	fats;		/* 10: number of FATs */
-	__u16	dir_entries;	/* 11: number of root directory entries */
-	__u16	sectors;	/* 13: if 0, total_sect supersedes */
-	__u8	media;		/* 15: media code */
-	__u16	fat_length;	/* 16: sectors/FAT for FAT12/16 */
-	__u16	secs_track;	/* 18: sectors per track */
-	__u16	heads;		/* 1a: number of heads */
-	__u32	hidden;		/* 1c: hidden sectors (partition start) */
-	__u32	sector_count;	/* 20: no. of sectors (if sectors == 0) */
+	uint8_t		boot_jump[3];	/* 00: Boot strap short or near jump */
+	uint8_t		system_id[8];	/* 03: system name */
+	uint16_t	sector_size;	/* 0b: bytes per logical sector */
+	uint8_t		cluster_size;	/* 0d: sectors/cluster */
+	uint16_t	reserved;	/* 0e: reserved sectors */
+	uint8_t		fats;		/* 10: number of FATs */
+	uint16_t	dir_entries;	/* 11: number of root directory entries */
+	uint16_t	sectors;	/* 13: if 0, total_sect supersedes */
+	uint8_t		media;		/* 15: media code */
+	uint16_t	fat_length;	/* 16: sectors/FAT for FAT12/16 */
+	uint16_t	secs_track;	/* 18: sectors per track */
+	uint16_t	heads;		/* 1a: number of heads */
+	uint32_t	hidden;		/* 1c: hidden sectors (partition start) */
+	uint32_t	sector_count;	/* 20: no. of sectors (if sectors == 0) */
 
 union {
 /* FAT16 fields */
 struct {
-	__u8	drive_num;	/* 24: */
-	__u8	empty_1;	/* 25: */
-	__u8	ext_signature;	/* 26: always 0x29 */
-	__u32	serial_number;	/* 27: */
-	__u8	volume_name [11];       /* 2b: */
-	__u8	fat_name [8];	/* 36: */
+	uint8_t		drive_num;	/* 24: */
+	uint8_t		empty_1;	/* 25: */
+	uint8_t		ext_signature;	/* 26: always 0x29 */
+	uint32_t	serial_number;	/* 27: */
+	uint8_t		volume_name [11];       /* 2b: */
+	uint8_t		fat_name [8];	/* 36: */
 
-	__u8	boot_code[448];	/* 3f: Boot code (or message) */
+	uint8_t		boot_code[448];	/* 3f: Boot code (or message) */
 } __attribute__ ((packed)) fat16;
 
 /* FAT32 fields */
 struct {
-	__u32	fat_length;	/* 24: size of FAT in sectors */
-	__u16	flags;		/* 28: bit8: fat mirroring, low4: active fat */
-	__u16	version;        /* 2a: minor * 256 + major */
-	__u32	root_dir_cluster;	/* 2c: */
-	__u16	info_sector;    /* 30: */
-	__u16	backup_sector;	/* 32: */
+	uint32_t	fat_length;	/* 24: size of FAT in sectors */
+	uint16_t	flags;		/* 28: bit8: fat mirroring, low4: active fat */
+	uint16_t	version;        /* 2a: minor * 256 + major */
+	uint32_t	root_dir_cluster;	/* 2c: */
+	uint16_t	info_sector;    /* 30: */
+	uint16_t	backup_sector;	/* 32: */
 
-	__u8	empty_1 [12];	/* 34: */
+	uint8_t		empty_1 [12];	/* 34: */
 
-	__u16	drive_num;	/* 40: */
-	__u8	ext_signature;	/* 42: always 0x29 */
-	__u32	serial_number;	/* 43: */
-	__u8	volume_name [11];	/* 47: */
-	__u8	fat_name [8];	/* 52: */
+	uint16_t	drive_num;	/* 40: */
+	uint8_t		ext_signature;	/* 42: always 0x29 */
+	uint32_t	serial_number;	/* 43: */
+	uint8_t		volume_name [11];	/* 47: */
+	uint8_t		fat_name [8];	/* 52: */
 
-	__u8	boot_code[420];	/* 5a: Boot code (or message) */
+	uint8_t		boot_code[420];	/* 5a: Boot code (or message) */
 } __attribute ((packed)) fat32;
 } u;
 
-	__u16	boot_sign;	/* 1fe: always 0xAA55 */
+	uint16_t	boot_sign;	/* 1fe: always 0xAA55 */
 } __attribute__ ((packed));
 
 struct _FatInfoSector {
-	__u32	signature_1;	/* should be 0x41615252 */
-	__u8	unused [480];
-	__u32	signature_2;	/* should be 0x61417272 */
-	__u32	free_clusters;
-	__u32	next_cluster;	/* most recently allocated cluster */
-	__u8	unused2 [0xe];
-	__u16	signature_3;	/* should be 0xaa55 */
+	uint32_t	signature_1;	/* should be 0x41615252 */
+	uint8_t		unused [480];
+	uint32_t	signature_2;	/* should be 0x61417272 */
+	uint32_t	free_clusters;
+	uint32_t	next_cluster;	/* most recently allocated cluster */
+	uint8_t		unused2 [0xe];
+	uint16_t	signature_3;	/* should be 0xaa55 */
 } __attribute__ ((packed));
 
 int fat_boot_sector_read (FatBootSector* bs, const PedGeometry* geom);
+FatType fat_boot_sector_probe_type (const FatBootSector* bs,
+				    const PedGeometry* geom);
 int fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs);
 int fat_boot_sector_set_boot_code (FatBootSector* bs);
 int fat_boot_sector_generate (FatBootSector* bs, const PedFileSystem* fs);
