@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -164,6 +164,7 @@ struct hd_driveid {
 #define BLKSSZGET  _IO(0x12,104) /* get block device sector size */
 #define BLKGETLASTSECT  _IO(0x12,108) /* get last sector of block device */
 #define BLKSETLASTSECT  _IO(0x12,109) /* set last sector of block device */
+#define BLKGETSIZE64 _IOR(0x12,114,sizeof(uint64_t))	/* return device size in bytes (u64 *arg) */
 
 struct blkdev_ioctl_param {
 	unsigned int block;
@@ -381,15 +382,33 @@ _device_get_sector_size (PedDevice* dev)
 	return sector_size;
 }
 
+static int
+_kernel_has_blkgetsize64(void)
+{
+	int version = _get_linux_version();
+
+	if (version >= KERNEL_VERSION (2,5,4)) return 1;
+	if (version <  KERNEL_VERSION (2,5,0) &&
+	    version >= KERNEL_VERSION (2,4,18)) return 1;
+        return 0;
+}
+
 /* TODO: do a binary search if BLKGETSIZE doesn't work?! */
 static PedSector
 _device_get_length (PedDevice* dev)
 {
 	unsigned long		size;
 	LinuxSpecific*		arch_specific = LINUX_SPECIFIC (dev);
+	uint64_t bytes=0;
 
 	PED_ASSERT (dev->open_count > 0, return 0);
 
+        if (_kernel_has_blkgetsize64()) {
+                if (ioctl(arch_specific->fd, BLKGETSIZE64, &bytes) == 0) {
+                        return bytes / _device_get_sector_size(dev);
+		}
+	}
+
 	if (ioctl (arch_specific->fd, BLKGETSIZE, &size)) {
 		ped_exception_throw (
 			PED_EXCEPTION_BUG,
@@ -653,7 +672,16 @@ init_file (PedDevice* dev)
 		dev->length = _device_get_length (dev);
 	else
 		dev->length = dev_stat.st_size / 512;
-	PED_ASSERT (dev->length > 0, goto error_close_dev);
+	if (dev->length <= 0) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			_("The device %s is zero-length, and can't possibly "
+			  "store a file system or partition table.  Perhaps "
+			  "you selected the wrong device?"),
+			dev->path);
+		goto error_close_dev;
+	}
 
 	ped_device_close (dev);
 
@@ -1293,6 +1321,45 @@ linux_sync (PedDevice* dev)
 	return 1;
 }
 
+static inline int
+_compare_digit_state (char ch, int need_digit)
+{
+	return !!isdigit (ch) == need_digit;
+}
+
+/* matches the regexp "[^0-9]+[0-9]+[^0-9]+[0-9]+$".
+ * Motivation: accept devices looking like /dev/rd/c0d0, but
+ * not looking like /dev/hda1 and /dev/rd/c0d0p1
+ */
+static int
+_match_rd_device (const char* name)
+{
+	const char* pos;
+	int state;
+
+	/* exclude directory names from test */
+	pos = strrchr(name, '/') ?: name;
+
+	/* states:
+	 * 	0	non-digits
+	 * 	1	digits
+	 * 	2	non-digits
+	 * 	3	digits
+	 */
+	for (state = 0; state < 4; state++) {
+		int want_digits = (state % 2 == 1);
+		do {
+			if (!*pos)
+				return 0;
+			if (!_compare_digit_state (*pos, want_digits))
+				return 0;
+			pos++;
+		} while (_compare_digit_state (*pos, want_digits));
+	}
+
+	return *pos == 0;
+}
+
 static int
 _probe_proc_partitions ()
 {
@@ -1315,7 +1382,8 @@ _probe_proc_partitions ()
 		/* Heuristic for telling partitions and devices apart
 		 * Probably needs to be improved
 		 */
-		if (isdigit (part_name [strlen (part_name) - 1]))
+		if (!_match_rd_device (part_name)
+		    && isdigit (part_name [strlen (part_name) - 1]))
 			continue;
 
 		strcpy (dev_name, "/dev/");
@@ -1384,7 +1452,8 @@ _device_get_part_path (PedDevice* dev, int num)
 	        snprintf (result + len, 16, "/part%d", num);
 	} else if (dev->type == PED_DEVICE_DAC960
 			|| dev->type == PED_DEVICE_CPQARRAY
-			|| dev->type == PED_DEVICE_ATARAID)
+			|| dev->type == PED_DEVICE_ATARAID
+			|| isdigit (dev->path[strlen(dev->path) - 1]))
 	        snprintf (result, result_len, "%sp%d", dev->path, num);
 	else
 	        snprintf (result, result_len, "%s%d", dev->path, num);
@@ -1407,18 +1476,21 @@ _partition_is_root_device (const char* part_name)
 	struct stat	part_stat;
 	FILE*		proc_real_root_dev;
 
+	/* even though this is an error, it probably means the partition
+	 * doesn't exist, etc., and therefore isn't root ;)
+	 */
 	if (stat (part_name, &part_stat))
-		goto error;
+		return 0;
 
 	proc_real_root_dev = fopen ("/proc/sys/kernel/real-root-dev", "r");
-	if (!proc_real_root_dev)
-		goto error;
-	fscanf (proc_real_root_dev, "%d", &root_dev);
-	fclose (proc_real_root_dev);
-	return root_dev == part_stat.st_rdev;
+ 	if (!proc_real_root_dev)
+ 		goto error;
+ 	fscanf (proc_real_root_dev, "%d", &root_dev);
+ 	fclose (proc_real_root_dev);
+ 	return root_dev == part_stat.st_rdev;
 
 error:
-	return 0;
+	return -1;
 }
 
 /* returns:
@@ -1449,7 +1521,8 @@ _mount_table_search (const char* mtab_path, const char* part_name)
 				goto error_close_mtab;
 			}
 		}
-		if (strncmp (line, part_name, strlen (part_name)) == 0) {
+		if (strncmp (line, part_name, strlen (part_name)) == 0
+		    && isspace (line [strlen (part_name)])) {
 			fclose (mtab);
 			return 1;
 		}
@@ -1471,29 +1544,39 @@ error:
 static int
 _partition_is_mounted_by_path (const char* path)
 {
-	int	status;
+	static int can_probe = 1;
 
-	if (_partition_is_root_device (path))
-		return 1;
+	if (!can_probe)
+		return 0;
 
-	status = _mount_table_search ("/proc/mounts", path);
-	if (status == 1)
-		return 1;
-	if (status == -1) {
-		status = _mount_table_search ("/etc/mtab", path);
-		if (status == -1)
-			return status;
+	switch (_partition_is_root_device (path)) {
+		case 1: return 1;
+		case -1: goto unknown;
 	}
+	switch (_mount_table_search ("/proc/mounts", path)) {
+		case 1: return 1;
+		case -1:
+			switch (_mount_table_search ("/etc/mtab", path)) {
+				case 1: return 1;
+				case -1: goto unknown;
+			}
+	}
+	switch (_mount_table_search ("/proc/swaps", path)) {
+		case 1: return 1;
+		case -1: goto unknown;
+	}
+	return 0;
 
-	status = _mount_table_search ("/proc/swaps", path);
-	if (status != -1)
-		return status;
-
+unknown:
+	can_probe = 0;
 	return ped_exception_throw (
 			PED_EXCEPTION_WARNING,
 			PED_EXCEPTION_IGNORE_CANCEL,
-			_("Unable to determine if partition is mounted."))
-				!= PED_EXCEPTION_IGNORE;
+			_("Unable to determine if partitions are mounted via "
+			  "/proc/mounts or /etc/mtab.  Make sure you don't "
+			  "attempt to resize or modify mounted file systems.  "
+			  "(Even read-only mounted)"))
+		!= PED_EXCEPTION_IGNORE;
 }
 
 static int
@@ -1618,8 +1701,8 @@ _disk_sync_part_table (PedDisk* disk)
 	int	ret = 1;
 
 	for (i = 1; i <= last; i++) {
-		rets[i] = _blkpg_remove_partition (disk, i);
-		errnums[i] = errno;
+		rets[i - 1] = _blkpg_remove_partition (disk, i);
+		errnums[i - 1] = errno;
 	}
 
 	for (i = 1; i <= last; i++) {
@@ -1638,7 +1721,7 @@ _disk_sync_part_table (PedDisk* disk)
 			 * doesn't matter anyway, because users shouldn't be
 			 * changing mounted partitions anyway...
 			 */
-			if (!rets[i] && errnums[i] == EBUSY)
+			if (!rets[i - 1] && errnums[i - 1] == EBUSY)
 					continue;
 
 			/* add the (possibly modified or new) partition */
