@@ -1,6 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -84,7 +84,8 @@ static char* flag_msg_start =	N_("FLAG is one of: ");
 static char* part_type_msg =	N_("PART-TYPE is one of: primary, logical, "
 			           "extended\n");
 static char* fs_type_msg_start = N_("FS-TYPE is one of: ");
-static char* start_end_msg =	N_("START and END are in megabytes\n");
+static char* start_end_msg =	N_("START and END are in megabytes.  "
+		"Negative values count from the end of the disk.\n");
 static char* state_msg =	N_("STATE is one of: on, off\n");
 static char* device_msg =	N_("DEVICE is usually /dev/hda or /dev/sda\n");
 static char* name_msg =		N_("NAME is any word you want\n");
@@ -140,7 +141,8 @@ _partition_warn_busy (PedPartition* part)
 		if (ped_exception_throw (
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_IGNORE_CANCEL,
-			_("Partition %s is being used."),
+			_("Partition %s is being used.  Modifying it while it "
+			  "is in use could cause severe corruption."),
 			path)
 				!= PED_EXCEPTION_IGNORE)
 			goto error_free_path;
@@ -892,7 +894,7 @@ do_print (PedDevice** dev)
 
 		if (has_extended)
 			printf ("%-9s ",
-				ped_partition_type_get_name (part->type));
+				_(ped_partition_type_get_name (part->type)));
 
 		printf ("%-12s", part->fs_type ? part->fs_type->name : "");
 
